from abc import ABCMeta, abstractmethod

from commons.util import Translator


class Formatter(object):
    _metaclass__ = ABCMeta

    @abstractmethod
    def format_report(self, pick_list_dto, translator):
        # type: (object, Translator) -> str

        raise NotImplementedError()
