from ._ApplyCouponContext import ApplyCouponContext


class ApplyCouponRequest(object):
    def __init__(
        self,
        coupon_id,  # type: str
        context,    # type: ApplyCouponContext
    ):
        # type: (...) -> None

        self.coupon_id = coupon_id
        self.context = context
