from abc import ABCMeta, abstractmethod

from model.electronic_message import ElectronicMessage


class ApiServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def send_electronic_messages(self, electronic_messages):
        # type: (ElectronicMessage) -> None

        raise NotImplementedError
