from model.loyalty import Benefit


class BenefitInfo(object):
    def __init__(
        self,
        notify,     # type: bool
        benefit,    # type: Benefit
    ):
        # type: (...) -> None

        self.notify = notify
        self.benefit = benefit
