from abc import (
    ABCMeta,
    abstractmethod
)
from typing import Any


class NavigationServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_navigation_descriptions(
        self,
    ):
        # type: (...) -> Any

        raise NotImplementedError
