# encoding: utf-8
from ._JustificationHeaderDto import JustificationHeaderDto
from ._JustificationBodyDto import JustificationBodyDto


class JustificationGeneratorDto(object):
    def __init__(
        self,
        header,     # type: JustificationHeaderDto
        body,       # type: JustificationBodyDto
    ):
        # type: (...) -> None

        self.header = header
        self.body = body
