class ListenedEvents(object):

    # ElectronicPayment
    ELECTRONIC_PAYMENT_SHOW_CHOICES = "SHOW_CHOICES"
    ELECTRONIC_PAYMENT_SHOW_MESSAGE = "SHOW_MESSAGE"
    ELECTRONIC_PAYMENT_SHOW_QUESTION = "SHOW_QUESTION"
    ELECTRONIC_PAYMENT_SHOW_QRCODE = "SHOW_QRCODE"
    ELECTRONIC_PAYMENT_SHOW_INPUT = "SHOW_INPUT"
    ELECTRONIC_PAYMENT_CLOSE_DIALOGS = "CLOSE_DIALOGS"
