from httpmessageprocessorutil.automapped import auto_map
from typing import Optional


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str},
           "price_list": {"name": "priceList", "type": Optional[str]},
           "sale_type": {"name": "saleType", "type": Optional[str]}})
class OrderCreateRequest(object):
    def __init__(self, system_api_key, pos_id, price_list="", sale_type=""):
        # type: (str, str, Optional[str], Optional[str]) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id
        self.price_list = price_list
        self.sale_type = sale_type
