from typing import Optional


class Configs(object):

    def __init__(
        self,
        loader_path=None,            # type: Optional[str]
        databases_path=None,         # type: Optional[str]
        backup_databases_path=None,  # type: Optional[str]
        component_name=None,         # type: Optional[str]
        required_services=None,      # type: Optional[str]
        instances=None,              # type: Optional[int]
        custom_data=None,            # type: Optional[str]
    ):
        # type: (...) -> None

        self.loader_path = loader_path
        self.databases_path = databases_path
        self.backup_databases_path = backup_databases_path
        self.component_name = component_name
        self.instances = instances
        self.required_services = required_services
        self.custom_data = custom_data
        self.alerts = None
