class Responses(object):

    ValidationError = 422

    BadRequestError = 400

    AuthValidation = 401

    NotFoundError = 404

    ClientCustomError = 406

    BlockedError = 409

    InUseError = 410

    NotAvailableError = 410

    OperationErrors = 500

    ExternalServicesError = 503
