from httpmessageprocessorutil.automapped import auto_map
from model.electronic_message import ElectronicMessage


@auto_map({"electronic_message": {"name": "electronicMessage", "type": ElectronicMessage}})
class CacheRequest(object):
    def __init__(self, electronic_message):
        # type: (ElectronicMessage) -> None

        self.electronic_message = electronic_message
