from ._CustomException import (
    ConfigurationReaderException,
    NotValidBenefit,
    ErrorLockingCoupon,
    ErrorUnlockingCoupon,
    ErrorBurningCoupon,
    ErrorObtainingCouponInfo,
    ErrorApplyingBenefit,
    BenefitIdNotFound,
    ErrorCheckingVoucher,
)
