from helper import ExtendedEnum


class NotMappedEvents(ExtendedEnum):

    ORDER_MODIFIED = "ORDER_MODIFIED_"
    ORDER_MODIFIED_CLEAR_TENDERS = ORDER_MODIFIED + "CLEAR_TENDERS"
    ORDER_MODIFIED_STORED = ORDER_MODIFIED + "STORED"
    ORDER_MODIFIED_RESET_CURRENT_ORDER = ORDER_MODIFIED + "RESET_CURRENT_ORDER"
    ORDER_MODIFIED_DO_SALE = ORDER_MODIFIED + "DO_SALE"
    ORDER_MODIFIED_CREATE_ORDER = ORDER_MODIFIED + "CREATE_ORDER"
    ORDER_MODIFIED_DO_TENDER = ORDER_MODIFIED + "DO_TENDER"
    ORDER_MODIFIED_CUSTOM_PROPERTY = ORDER_MODIFIED + "CUSTOM_PROPERTY"
    ORDER_MODIFIED_ORDER_PROPERTIES_CHANGED = ORDER_MODIFIED + "ORDER_PROPERTIES_CHANGED"
    ORDER_MODIFIED_CHANGE_DIMENSION = ORDER_MODIFIED + "CHANGE_DIMENSION"
    ORDER_MODIFIED_RECALLED = ORDER_MODIFIED + "RECALLED"
    ORDER_MODIFIED_DISCOUNT_APPLIED = ORDER_MODIFIED + "DISCOUNT_APPLIED"
    ORDER_MODIFIED_DISCOUNT_REMOVED = ORDER_MODIFIED + "DISCOUNT_REMOVED"
