from ._LoyaltyToken import LoyaltyToken
from messagebus import TokenCreator, TokenPriority


class LoyaltyRequestToken(LoyaltyToken):
    TK_LOYALTY_GET_AND_LOCK_VOUCHER = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
        number="2"
    )
    TK_LOYALTY_BURN_VOUCHER = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
        number="3"
    )
    TK_LOYALTY_UNLOCK_VOUCHER = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
        number="4"
    )
