from typing import Any

from messageprocessorutil import UuidMessageProcessor


class NotMappedProcessor(UuidMessageProcessor):
    def __init__(self):
        # type: () -> None

        super(NotMappedProcessor, self).__init__()

    def parse_data(self, data, event_type):
        # type: (str, str) -> None

        return

    def call_business(self, param):
        # type: (Any) -> None

        return

    def format_response(self, message_bus, message, event, request, response):
        return

    def format_exception(self, message_bus, message, event, input_obj, exception):
        return

    def format_parse_exception(self, message_bus, message, event, data, exception):
        return
