from helper import ExtendedEnum
from ._LoyaltyToken import LoyaltyToken


class ResponseToken(ExtendedEnum):
    TK_LOYALTY_ID_NOT_FOUND = LoyaltyToken.create_token(number="F001")
    TK_LOYALTY_ERROR = LoyaltyToken.create_token(number="F002")
